"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeMapping_1 = require("./typeMapping");
function parseStruct(codes, options = {}) {
    const name = /^struct\s+([\w\d_]+)(?:\s+)?{/.exec(codes[0].trim())[1];
    const properties = {};
    const propertyLines = codes
        .map(d => d.trim())
        .filter(d => !!d)
        .slice(1, codes.length - 1);
    for (let i = 0; i < propertyLines.length; i++) {
        const line = propertyLines[i];
        const [index, ...rest] = line.split(':').map(d => d.trim());
        const partialRight = rest.join(':');
        if (isNaN(Number(index))) {
            // 如果没有 fieldIndex，可能是注释行，跳过
            continue;
        }
        /**
         * 为了防止以后看不懂，详解如下
         *    - $1 (?:(optional|required)\s+)? 捕获 optional 或 required
         *    - $2 ([\w\d_>.,\s<]+) 捕获类型
         *    - $3 ([\w\d_]+)(?:\(.+\))? 捕获字段名
         *    - $4 (\(.+\)) 捕获括号附加信息
         *    - $5 (?:\s*=\s*([\d\.\w_-]+|true|false|"[^"]*"|'[^']*'))? 捕获默认值
         *        - [\d\.\w_-]+             数字，enum？（不确定 thrift 是否支持 EnumType.XXX 作为默认值）
         *        - true|false              顾名思义
         *        - "[^"]*"|'[^']*'         匹配字符串
         *    - $6 (?:(?:\s+)?(?:\/{2,}|#{1,})(.+))? 捕获注释
         */
        const mc = /(?:(optional|required)\s+)?([\w\d_>.,\s<]+)\s+([\w\d_]+)\s*(\(.+\))?(?:\s*=\s*([\d\.\w_-]+|true|false|"[^"]*"|'[^']*'))?\s*[,;]?(?:(?:\s+)?(?:\/{2,}|#{1,})(.+))?$/.exec(partialRight);
        /* istanbul ignore if */
        if (mc === null) {
            continue;
        }
        const type = typeMapping_1.typeMapping(mc[2].trim());
        let key = mc[3].trim();
        const extra = mc[4];
        const defaultValue = (mc[5] || '').trim();
        const comment = (mc[6] || '').trim();
        let optional = options.useStrictMode
            ? mc[1] !== 'required'
            : !(mc[1] !== 'optional');
        if (defaultValue) {
            // 如果有默认值，不需要指定 optional
            optional = false;
        }
        if (options.useTag && extra && extra.match(/(\w+).tag="json:\\"([\w_\d]+).*\\"/)) {
            if (RegExp.$1 === options.useTag) {
                key = RegExp.$2;
            }
        }
        properties[key] = {
            optional,
            comment,
            type,
            index: Number(index),
            defaultValue
        };
    }
    return {
        name,
        properties,
        childrenInterfaces: [],
        childrenEnums: []
    };
}
exports.parseStruct = parseStruct;
